window.Gleam.initSearch([{"doc":"glove","title":"glove","content":"\n&lt;img src=&quot;https://github.com/Willyboar/glove/assets/22755228/bfd9673f-a105-4083-82d8-84bf4b0ff071&quot;&gt;\n\n\n[![Package Version](https://img.shields.io/hexpm/v/glove)](https://hex.pm/packages/glove)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glove/)\n\n\n## About \nGlove is a library for working with QBE intermediate representation (IR) in Gleam. Provides utilities and functions to generate QBE code using the Gleam programming language. \n\n## Requirements\n\nTo use Glove, you need to have the following dependencies installed:\n\nGleam programming language - Install instructions can be found [here](https://gleam.run/getting-started/installing/)\n\nQBE Backend can be found [here](https://c9x.me/compile/)\n\n\n## Quick start\n\nTODO\n\n## Run the tests\n```sh\ngleam test  # Run the tests\n```\n\n## Installation\n\nIf available on Hex this package can be added to your Gleam project:\n\n```sh\ngleam add glove\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/glove&gt;.\n","url":"index.html"},{"doc":"glove","title":"Block","content":"pub type Block {\n  Block(label: String, statements: List(Statement))\n}\n Function block with a label\nBlock(label: String, statements: List(Statement))\n\n\nSynonyms:\nglove.Block\nglove Block","url":"glove.html#Block"},{"doc":"glove","title":"Comp","content":"pub type Comp {\n  Slt\n  Sle\n  Sgt\n  Sge\n  Eq\n  Ne\n}\n QBE Comparison Operators\nSlt\n Less Than\n\nSle\n Less or Equal\n\nSgt\n Greater than\n\nSge\n Greater or equal\n\nEq\n Equal\n\nNe\n Not equal\n\nSynonyms:\nglove.Comp\nglove Comp","url":"glove.html#Comp"},{"doc":"glove","title":"DataDef","content":"pub type DataDef {\n  DataDef(\n    linkage: Linkage,\n    name: String,\n    align: Option(Int),\n    items: List(#(Type, DataItem)),\n  )\n}\n QBE data definition\nDataDef(\n  linkage: Linkage,\n  name: String,\n  align: Option(Int),\n  items: List(#(Type, DataItem)),\n)\n\n\nSynonyms:\nglove.DataDef\nglove DataDef","url":"glove.html#DataDef"},{"doc":"glove","title":"DataItem","content":"pub type DataItem {\n  Symbol(String, Option(Int))\n  Str(String)\n  Constant(Int)\n}\n QBE Data definition item\nSymbol(String, Option(Int))\n Symbol and offset\n\nStr(String)\n String\n\nConstant(Int)\n Integer\n\nSynonyms:\nglove.DataItem\nglove DataItem","url":"glove.html#DataItem"},{"doc":"glove","title":"Function","content":"pub type Function {\n  Function(\n    linkage: Linkage,\n    name: String,\n    arguments: List(#(Type, Value)),\n    return_ty: Option(Type),\n    blocks: List(Block),\n  )\n}\n QBE Function\nFunction(\n  linkage: Linkage,\n  name: String,\n  arguments: List(#(Type, Value)),\n  return_ty: Option(Type),\n  blocks: List(Block),\n)\n\n\nSynonyms:\nglove.Function\nglove Function","url":"glove.html#Function"},{"doc":"glove","title":"Inst","content":"pub type Inst {\n  Add(Value, Value)\n  Sub(Value, Value)\n  Mul(Value, Value)\n  Div(Value, Value)\n  Rem(Value, Value)\n  Comp(Type, Comp, Value, Value)\n  And(Value, Value)\n  Or(Value, Value)\n  Copy(Value)\n  Ret(Option(Value))\n  Jnz(Value, String, String)\n  Jmp(String)\n  Call(Value, List(#(Type, Value)))\n  Alloc4(Int)\n  Alloc8(Int)\n  Alloc16(Int)\n  Store(Type, Value, Value)\n  Load(Type, Value)\n  Blit(Value, Value, Int)\n}\n QBE instruction\nAdd(Value, Value)\n Adds values \n\nSub(Value, Value)\n Substracts value(b) from value(a)\n\nMul(Value, Value)\n Multiplies values \n\nDiv(Value, Value)\n Divides value(a) by value(b)\n\nRem(Value, Value)\n Returns a remaider from division\n\nComp(Type, Comp, Value, Value)\n Perform a Comparison\n\nAnd(Value, Value)\n Bitwise AND\n\nOr(Value, Value)\n Bitwise OR\n\nCopy(Value)\n Copies either temporary or literal value\n\nRet(Option(Value))\n Return from a function, optionally with a value\n\nJnz(Value, String, String)\n Jumps to first label if a value is nonzero or to the second one otherwise\n\nJmp(String)\n Unconditionally jumps to a label\n\nCall(Value, List(#(Type, Value)))\n Calls a function\n\nAlloc4(Int)\n Allocates a 4-byte aligned area on the stack\n\nAlloc8(Int)\n Allocates a 8-byte aligned area on the stack\n\nAlloc16(Int)\n Allocates a 16-byte aligned area on the stack\n\nStore(Type, Value, Value)\n Stores a value into memory pointed to by destination.\n (type, destination, value)\n\nLoad(Type, Value)\n Loads a value from memory pointed to by source\n (type, source)\n\nBlit(Value, Value, Int)\n (source, destination, n)\n\n Copy `n` bytes from the source address to the destination address.\n\n n must be a constant value.\n\n Minimum supported QBE version 1.1\n\nSynonyms:\nglove.Inst\nglove Inst","url":"glove.html#Inst"},{"doc":"glove","title":"Linkage","content":"pub type Linkage {\n  Linkage(\n    exported: Bool,\n    section: Option(String),\n    secflags: Option(String),\n  )\n}\n Linkage of a function or data defintion (e.g. section and\n private/public status)\nLinkage(\n  exported: Bool,\n  section: Option(String),\n  secflags: Option(String),\n)\n\n\nSynonyms:\nglove.Linkage\nglove Linkage","url":"glove.html#Linkage"},{"doc":"glove","title":"Module","content":"pub type Module {\n  Module(\n    functions: List(Function),\n    types: List(TypeDef),\n    data: List(DataDef),\n  )\n}\n A complete IL file \nModule(\n  functions: List(Function),\n  types: List(TypeDef),\n  data: List(DataDef),\n)\n\n\nSynonyms:\nglove.Module\nglove Module","url":"glove.html#Module"},{"doc":"glove","title":"Statement","content":"pub type Statement {\n  Assign(Value, Type, Inst)\n  Volatile(Inst)\n}\n IR Statement\nAssign(Value, Type, Inst)\n\n\nVolatile(Inst)\n\n\nSynonyms:\nglove.Statement\nglove Statement","url":"glove.html#Statement"},{"doc":"glove","title":"Type","content":"pub type Type {\n  Word\n  Long\n  Single\n  Double\n  Byte\n  Halfword\n  Aggregate(TypeDef)\n}\n QBE Types\nWord\n Base Types\n\nLong\n\n\nSingle\n\n\nDouble\n\n\nByte\n Extended Types\n\nHalfword\n\n\nAggregate(TypeDef)\n\n\nSynonyms:\nglove.Type\nglove Type","url":"glove.html#Type"},{"doc":"glove","title":"TypeDef","content":"pub type TypeDef {\n  TypeDef(\n    name: String,\n    align: Option(Int),\n    items: List(#(Type, Int)),\n  )\n}\n QBE aggregate type definition\nTypeDef(\n  name: String,\n  align: Option(Int),\n  items: List(#(Type, Int)),\n)\n\n\nSynonyms:\nglove.TypeDef\nglove TypeDef","url":"glove.html#TypeDef"},{"doc":"glove","title":"Value","content":"pub type Value {\n  Temporary(name: String)\n  Global(name: String)\n  Const(value: Int)\n}\n QBE Value for instructions\nTemporary(name: String)\n `%`-temporary\n\nGlobal(name: String)\n `$`-global\n\nConst(value: Int)\n Constant\n\nSynonyms:\nglove.Value\nglove Value","url":"glove.html#Value"},{"doc":"glove","title":"add_block","content":"pub fn add_block(label: String) -&gt; Block\n Adds a new empty block with a specified label and returns \n a reference to it\nSynonyms:\nglove.add_block\nglove add_block","url":"glove.html#add_block"},{"doc":"glove","title":"add_data","content":"pub fn add_data(module: Module, data_def: DataDef) -&gt; Module\n Add Data to module\nSynonyms:\nglove.add_data\nglove add_data","url":"glove.html#add_data"},{"doc":"glove","title":"add_function","content":"pub fn add_function(module: Module, function: Function) -&gt; Module\n Add function to module\nSynonyms:\nglove.add_function\nglove add_function","url":"glove.html#add_function"},{"doc":"glove","title":"add_inst","content":"pub fn add_inst(block: Block, inst: Inst) -&gt; Block\n Adds a new instruction to the block\nSynonyms:\nglove.add_inst\nglove add_inst","url":"glove.html#add_inst"},{"doc":"glove","title":"add_type","content":"pub fn add_type(module: Module, type_def: TypeDef) -&gt; Module\n Add type to module\nSynonyms:\nglove.add_type\nglove add_type","url":"glove.html#add_type"},{"doc":"glove","title":"assign_inst","content":"pub fn assign_inst(block: Block, val: Value, typ: Type, inst: Inst) -&gt; Block\n Adds a new instruction assigned to a temporary\nSynonyms:\nglove.assign_inst\nglove assign_inst","url":"glove.html#assign_inst"},{"doc":"glove","title":"display_arguments","content":"pub fn display_arguments(arguments: List(#(Type, Value))) -&gt; String\n Display functions Arguments\nSynonyms:\nglove.display_arguments\nglove display_arguments","url":"glove.html#display_arguments"},{"doc":"glove","title":"display_block","content":"pub fn display_block(block: Block) -&gt; String\n Display function for block\nSynonyms:\nglove.display_block\nglove display_block","url":"glove.html#display_block"},{"doc":"glove","title":"display_blocks","content":"pub fn display_blocks(blocks: List(Block)) -&gt; String\n Display blocks\nSynonyms:\nglove.display_blocks\nglove display_blocks","url":"glove.html#display_blocks"},{"doc":"glove","title":"display_data_def","content":"pub fn display_data_def(def: DataDef) -&gt; String\n Display function for Datadef\nSynonyms:\nglove.display_data_def\nglove display_data_def","url":"glove.html#display_data_def"},{"doc":"glove","title":"display_data_item","content":"pub fn display_data_item(item: DataItem) -&gt; String\n Display function for DataItem\nSynonyms:\nglove.display_data_item\nglove display_data_item","url":"glove.html#display_data_item"},{"doc":"glove","title":"display_function","content":"pub fn display_function(func: Function) -&gt; String\n Display function for functions\nSynonyms:\nglove.display_function\nglove display_function","url":"glove.html#display_function"},{"doc":"glove","title":"display_inst","content":"pub fn display_inst(inst: Inst) -&gt; String\n Display function for Instructions\nSynonyms:\nglove.display_inst\nglove display_inst","url":"glove.html#display_inst"},{"doc":"glove","title":"display_linkage","content":"pub fn display_linkage(linkage: Linkage) -&gt; String\n Display function for Linkage\nSynonyms:\nglove.display_linkage\nglove display_linkage","url":"glove.html#display_linkage"},{"doc":"glove","title":"display_module","content":"pub fn display_module(module: Module) -&gt; String\n Display function for Module\nSynonyms:\nglove.display_module\nglove display_module","url":"glove.html#display_module"},{"doc":"glove","title":"display_statement","content":"pub fn display_statement(stmt: Statement) -&gt; String\n Display function for Statement \nSynonyms:\nglove.display_statement\nglove display_statement","url":"glove.html#display_statement"},{"doc":"glove","title":"display_type","content":"pub fn display_type(ty: Type) -&gt; String\n Display Type function\nSynonyms:\nglove.display_type\nglove display_type","url":"glove.html#display_type"},{"doc":"glove","title":"display_type_def","content":"pub fn display_type_def(def: TypeDef) -&gt; String\n Display function for TypeDef\nSynonyms:\nglove.display_type_def\nglove display_type_def","url":"glove.html#display_type_def"},{"doc":"glove","title":"display_value","content":"pub fn display_value(value: Value) -&gt; String\n Display Value function\nSynonyms:\nglove.display_value\nglove display_value","url":"glove.html#display_value"},{"doc":"glove","title":"into_abi","content":"pub fn into_abi(self: Type) -&gt; Type\n Aggregate type with a specified name\n Returns a C ABI type. Extended types are converted to closest base\n types\nSynonyms:\nglove.into_abi\nglove into_abi","url":"glove.html#into_abi"},{"doc":"glove","title":"into_base","content":"pub fn into_base(self: Type) -&gt; Type\n Returns the closest base type\nSynonyms:\nglove.into_base\nglove into_base","url":"glove.html#into_base"},{"doc":"glove","title":"jumps","content":"pub fn jumps(block: Block) -&gt; Bool\n Returns true if the block&#39;s last instruction is a jump\nSynonyms:\nglove.jumps\nglove jumps","url":"glove.html#jumps"},{"doc":"glove","title":"last_block","content":"pub fn last_block(blocks: List(Block)) -&gt; Option(Block)\n Returns a reference to the last block\nSynonyms:\nglove.last_block\nglove last_block","url":"glove.html#last_block"},{"doc":"glove","title":"new_datadef","content":"pub fn new_datadef() -&gt; DataDef\n\nSynonyms:\nglove.new_datadef\nglove new_datadef","url":"glove.html#new_datadef"},{"doc":"glove","title":"new_function","content":"pub fn new_function() -&gt; Function\n Instantiates an empty function and returns it\nSynonyms:\nglove.new_function\nglove new_function","url":"glove.html#new_function"},{"doc":"glove","title":"new_module","content":"pub fn new_module() -&gt; Module\n Creates a new module\nSynonyms:\nglove.new_module\nglove new_module","url":"glove.html#new_module"},{"doc":"glove","title":"private","content":"pub fn private() -&gt; Linkage\n Returns the default configuration for private linkage\nSynonyms:\nglove.private\nglove private","url":"glove.html#private"},{"doc":"glove","title":"private_with_section","content":"pub fn private_with_section(section: String) -&gt; Linkage\n Returns the configuration for private linkage with a provided section\nSynonyms:\nglove.private_with_section\nglove private_with_section","url":"glove.html#private_with_section"},{"doc":"glove","title":"public","content":"pub fn public() -&gt; Linkage\n Returns the default configuration for public linkage\nSynonyms:\nglove.public\nglove public","url":"glove.html#public"},{"doc":"glove","title":"public_with_section","content":"pub fn public_with_section(section: String) -&gt; Linkage\n Returns the configuration for public linkage with a provided section\nSynonyms:\nglove.public_with_section\nglove public_with_section","url":"glove.html#public_with_section"},{"doc":"glove","title":"size","content":"pub fn size(self: Type) -&gt; Int\n Returns byte size for values of the type\nSynonyms:\nglove.size\nglove size","url":"glove.html#size"},{"doc":"glove","title":"glove","content":"","url":"glove.html"}]);